/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "Screen.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomScreen.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomScreen.h"
#include "gdom/PrivateIntGdomScreen.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Screen* obj)
{
    return WebCore::cacheDOMObject<WebCore::Screen, GDOMScreen>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMScreen::GDOMScreen(PassRefPtr<WebCore::Screen> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMScreen::~GDOMScreen()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Screen.h"
#include <gdom/GdomScreen.h>
#include <gdom/PrivateGdomScreen.h>
#include <gdom/PrivateIntGdomScreen.h>
#include <gdom/GdomScreenprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomScreen, gdom_screen, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomScreenPrivate {
    class WebCore::Screen *item;
};

#define GDOM_SCREEN_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_SCREEN, GdomScreenPrivate))
*/
G_END_DECLS
WebCore::Screen *
gdom_screen_get_private(GdomScreen *request)
{
    GDOMScreen *res;
    res = static_cast<GDOMScreen*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Screen*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_HEIGHT,
    PROP_WIDTH,
    PROP_COLOR_DEPTH,
    PROP_PIXEL_DEPTH,
    PROP_AVAIL_LEFT,
    PROP_AVAIL_TOP,
    PROP_AVAIL_HEIGHT,
    PROP_AVAIL_WIDTH,
};


static void gdom_screen_finalize(GObject* object)
{
    /*GdomScreen* request = GDOM_SCREEN(object);*/
    /*GdomScreenPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_screen_parent_class)->finalize(object);
}

static void gdom_screen__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomScreen *self = GDOM_SCREEN(object);
    WebCore::Screen * gself = (gdom_screen_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_screen__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomScreen *self = GDOM_SCREEN(object);
    WebCore::Screen * gself = (gdom_screen_get_private(self));
    switch (prop_id) {
    case PROP_HEIGHT:
    {
         g_value_set_ulong (value, gself->height());
         break;
    }
    case PROP_WIDTH:
    {
         g_value_set_ulong (value, gself->width());
         break;
    }
    case PROP_COLOR_DEPTH:
    {
         g_value_set_ulong (value, gself->colorDepth());
         break;
    }
    case PROP_PIXEL_DEPTH:
    {
         g_value_set_ulong (value, gself->pixelDepth());
         break;
    }
    case PROP_AVAIL_LEFT:
    {
         g_value_set_ulong (value, gself->availLeft());
         break;
    }
    case PROP_AVAIL_TOP:
    {
         g_value_set_ulong (value, gself->availTop());
         break;
    }
    case PROP_AVAIL_HEIGHT:
    {
         g_value_set_ulong (value, gself->availHeight());
         break;
    }
    case PROP_AVAIL_WIDTH:
    {
         g_value_set_ulong (value, gself->availWidth());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_screen_class_init(GdomScreenClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_screen_finalize;
    g_klass->set_property = gdom_screen__set_property_fn;
    g_klass->get_property = gdom_screen__get_property_fn;

    g_object_class_install_property (g_klass, PROP_HEIGHT,
            g_param_spec_ulong ("height", /* name */
                "screen_height", /* short description */
                "read-only  gulong Screen.height", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_ulong ("width", /* name */
                "screen_width", /* short description */
                "read-only  gulong Screen.width", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_COLOR_DEPTH,
            g_param_spec_ulong ("color_depth", /* name */
                "screen_color_depth", /* short description */
                "read-only  gulong Screen.color_depth", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PIXEL_DEPTH,
            g_param_spec_ulong ("pixel_depth", /* name */
                "screen_pixel_depth", /* short description */
                "read-only  gulong Screen.pixel_depth", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_AVAIL_LEFT,
            g_param_spec_ulong ("avail_left", /* name */
                "screen_avail_left", /* short description */
                "read-only  gulong Screen.avail_left", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_AVAIL_TOP,
            g_param_spec_ulong ("avail_top", /* name */
                "screen_avail_top", /* short description */
                "read-only  gulong Screen.avail_top", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_AVAIL_HEIGHT,
            g_param_spec_ulong ("avail_height", /* name */
                "screen_avail_height", /* short description */
                "read-only  gulong Screen.avail_height", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_AVAIL_WIDTH,
            g_param_spec_ulong ("avail_width", /* name */
                "screen_avail_width", /* short description */
                "read-only  gulong Screen.avail_width", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomScreenPrivate));*/
}

static void gdom_screen_init(GdomScreen* request)
{
    /*GdomScreenPrivate* priv = GDOM_SCREEN_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

